import axios from 'axios'
import store from '@/store'
import { getToken } from '@/utils/auth'
import { blobToJson } from '@/utils'
import { checkToken } from '@/api/system'
import route from '@/router'
import qs from 'qs'
// import Vue from 'vue'
// create an axios instance
const service = axios.create({
  baseURL: window?.g?.VUE_APP_BASE_API ?? process.env.VUE_APP_BASE_API,
  // baseURL: process.env.VUE_APP_BASE_API, // url = base url + request url
  // withCredentials: true, // send cookies when cross-domain requests
  timeout: 120000 // request timeout
})

// request interceptor
// 发送请求拦截器
service.interceptors.request.use(
  config => {
    // do something before request is sent
    // if (getToken()) {
    //   config.headers.token = getToken()
    // }
    config.headers.token = getToken()
    return config
  },
  error => {
    // do something with request error
    // console.error(error) // for debug
    return Promise.reject(error)
  }
)

// response interceptor
// 请求返回之后的拦截器
service.interceptors.response.use(
  async response => {
    const res = response.data

    // console.log(response)

    if (response.request.responseType === 'blob') {
      // 返回的文件流当报错时转化成json
      if (response.headers['content-type'] === 'application/json') {
        try {
          const result = await blobToJson(res)
          return Promise.reject(result.msg)
        } catch (error) {
          return Promise.reject(error)
        }
      }
      const name = response.headers['content-disposition']
      // console.log(name)
      return {
        data: res,
        name: name ? decodeURI(name.replace('attachment;filename=', '')) : '未命名'
      }
    }

    // 多语言切分msg
    if (res.msg && typeof res.msg === 'string') {
      const msg = res.msg.split('|')
      if (msg.length > 0) {
        const list = ['zh', 'en']
        res.msg = msg[list.indexOf(store.getters.lang)]
      }
    }

    // 50008: Illegal token; 50012: Other clients logged in; 50014: Token expired;
    if ([50008, 50012, 50014, 402000, 401].includes(res.code)) {
      // Vue.prototype.$snackbar.error(res.msg)
      // if (!store.getters.singleSignOn) {
      //   // 删除token
      //   deleteToken()
      //   // route.push(`/login?redirect=${route.history.current.fullPath}`)
      //   window.location.href = `/login-local?redirect=${route.history.current.fullPath}`
      //   res.msg = '登陆过期，请重新登陆'
      //   return Promise.reject(res.msg)
      // }
      try {
        await checkToken()
      } catch (error) {
        if (!error?.data?.redirectUrl) return
        // window.location.href = error.data.redirectUrl
        const url = error.data.redirectUrl === 'local' ? `/login-local?redirect=${route.history.current.fullPath}` : error.data.redirectUrl
        window.location.href = url
      }
      const msg = '登陆过期，请重新登陆'
      return Promise.reject(msg)
    }
    // 登录验证码过期
    if (res.code === 60902) {
      return Promise.reject(res)
    }
    if (res.code !== 20000) {
      if (res.data && Object.keys(res.data).length) {
        return Promise.reject(res)
      }
      return Promise.reject(res.msg.error || res.msg)
    }
    return res
  },
  error => {
    // console.error(error)
    // error = error.message || error
    return Promise.reject(error)
  }
)

// 请求方法
const http = {
  post (url, params) {
    return service.post(url, params, {
      transformRequest: [(params) => {
        return JSON.stringify(params)
      }],
      headers: {
        'Content-Type': 'application/json'
      }
    })
  },
  get (url, params) {
    return service.get(url, {
      params: params,
      paramsSerializer: (params) => {
        return qs.stringify(params)
      }
    })
  },
  formData (url, params) {
    return service.post(url, params, {
      timeout: 600000,
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    })
  },
  upload (url, params) {
    return service.post(url, params, {
      timeout: 600000,
      headers: {
        'Content-Type': 'multipart/form-data'
      }
    })
  },
  download (url, params) {
    return service.post(url, params, {
      timeout: 10000,
      headers: {
        'Content-Type': 'application/json'
      },
      responseType: 'blob'
    })
  }
}
export default http
